import 'dart:io';
import 'package:bagel_db/bagel_db.dart';

import 'package:flutter_test/flutter_test.dart';

final String testToken = 'BearerTestToken';

getItemsTest() {
  group('get items test', () {
    // given
    BagelDB bagelDB = BagelDB(testToken);

    test('basic data fetching test', () async {
      // when
      BagelResponse response = await bagelDB.collection('testItems').get();
      var data = response.data;
      // then
      expect(response.statusCode, equals(200));
      expect(data[0]["name"], contains("Dov"));
    });

    test('basic query test', () async {
      BagelResponse response =
          await bagelDB.collection('testItems').query('name', '=', 'Dov').get();
      var data = response.data;
      // then
      expect(response.statusCode, equals(200));
      expect(data[0]["name"], contains("Dov"));
      expect(response.itemCount, 1);
    });

    test('multiple queries test', () async {
      BagelResponse response = await bagelDB
          .collection('testItems')
          .query('position', '=', 'developer')
          .query('age', '>', '27')
          .get();
      var data = response.data;
      // then
      expect(response.statusCode, equals(200));
      expect(data[0]["name"], contains("Nati"));
      expect(response.itemCount, 1);
    });

    test('sort test', () async {
      BagelResponse response =
          await bagelDB.collection('testItems').sortBy('age').get();
      var data = response.data;
      // then
      expect(response.statusCode, equals(200));
      expect(data[0]["name"], contains("Maayan"));
    });

    test('descending sort test', () async {
      BagelResponse response = await bagelDB
          .collection('testItems')
          .sortBy('age', orderTerm: 'desc')
          .get();
      var data = response.data;
      // then
      expect(response.statusCode, equals(200));
      expect(data[0]["name"], contains("Dov"));
    });

    test('ascending sort test', () async {
      BagelResponse response = await bagelDB
          .collection('testItems')
          .sortBy('age', orderTerm: 'asc')
          .get();
      var data = response.data;
      // then
      expect(response.statusCode, equals(200));
      expect(data[0]["name"], contains("Maayan"));
    });

    test('results per page test', () async {
      BagelResponse response =
          await bagelDB.collection('testItems').perPage(2).get();
      var data = response.data;
      // then
      expect(response.statusCode, equals(200));
      expect(data.length, lessThan(3));
    });

    test('everything success test', () async {
      BagelResponse response =
          await bagelDB.collection('testItems').everything().get().then((res) {
        print(res);
        return res;
      });
      var data = response.data;
      expect(response.statusCode, equals(200));
      expect(data[0]["seniority"][0]["item"]["seniority"], equals(21));
    });

    test('everything fail test', () async {
      BagelResponse falseResponse = await bagelDB.collection('testItems').get();
      var falseData = falseResponse.data;
      expect(falseResponse.statusCode, equals(200));
      expect(falseData[0]["seniority"][0]["item"], equals(null));
    });

    test('item test', () async {
      BagelResponse response = await bagelDB
          .collection('testItems')
          .item('5ee22f0e0a6090a13e2f777c')
          .get();
      var data = response.data;
      expect(response.statusCode, equals(200));
      //expect(data, null);
      expect(data["name"], contains('Nati'));
    });
  });
}

postItemTest() {
  group('post item test', () {
    // given
    BagelDB bagelDB = BagelDB(testToken);

    test('simple post test', () async {
      BagelResponse response =
          await bagelDB.collection('testItems').post(item: {
        'name': 'Renana',
        'age': 34,
        'position': 'CPO',
      });
      // then
      expect(response.statusCode, equals(201));
    });

    test('post with image test', () async {
      File file =
          File('/Users/rani/Desktop/photo-1593642632559-0c6d3fc62b89.jpeg');
      BagelResponse response =
          await bagelDB.collection('testItems').post(item: {
        'name': 'Renana',
        'age': 34,
        'position': 'COO',
      });
      String _id = response.data["id"];
      BagelResponse res = await bagelDB
          .collection("testItems")
          .item(_id)
          .uploadImage("image", file);
      // then
      expect(response.statusCode, equals(201));
      expect(res.statusCode, equals(201));
    });
  });
}

putItemTest() {
  group('put item test', () {
    // given
    BagelDB bagelDB = BagelDB(testToken);

    test('put test', () async {
      // when
      BagelResponse response = await bagelDB
          .collection('testItems')
          .item('5ee773795539f58b09d54447')
          .put(item: {
        'name': 'Renana',
        'age': 34,
        'position': 'Product Manager',
      });
      // then
      expect(response.statusCode, lessThan(300));
    });
  });
}

deleteItemTest() {
  group('delete item test', () {
    // given
    BagelDB bagelDB = BagelDB(testToken);

    test('delete test', () async {
      BagelResponse response =
          await bagelDB.collection('testItems').post(item: {
        'name': 'Renana',
        'age': 34,
        'position': 'CFO',
      });
      String _id = response.data["id"];
      BagelResponse res =
          await bagelDB.collection("testItems").item(_id).delete();
      expect(res.statusCode, equals(200));
    });
  });
}

void main() {
  getItemsTest();
  postItemTest();
  putItemTest();
  deleteItemTest();
}
